#include <RTClib.h>
#include <Arduino.h>
#include <BlynkSimpleEsp8266.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h> // Library for LCD; from fdebrabander; dfrobot
#ifdef ESP32
    #include <WiFi.h>
#else
    #include <ESP8266WiFi.h>
#endif

#include "AudioFileSourcePROGMEM.h"
#include "AudioGeneratorWAV.h"
#include "AudioOutputI2SNoDAC.h"
#include "viola.h"

#define BLYNK_PRINT Serial
 
RTC_DS3231 rtc;
LiquidCrystal_I2C lcd = LiquidCrystal_I2C(0x27, 16, 2); // Change to (0x27,20,4) for 20x4 LCD.

//authentication token for Blynk app
char auth[] = "";
// WiFi credentials.
char ssid[] = "";
char pass[] = "";

//To play audio
AudioGeneratorWAV *wav;
AudioFileSourcePROGMEM *file;
AudioOutputI2SNoDAC *out;

//String to display time
char t[32];
//To clear LCD and get rid of character noise
bool isCleared = true;

//Define variables to use millis() instead of delay
unsigned long previousMillis = 0;
unsigned long previousMillis2 = 0;
unsigned long previousMicros = 0;         //microsecond at which the pin was last writen
const long interval = 1000;
const long buttonInterval = 100;
const int triggerInterval1 = 2;                  //To set the trigPin LOW for 2 microseconds
const int triggerInterval2 = 12;                 //To set the trigPin HIGH for 10 microseconds

// defines pins numbers for ultrasonic sensor
const int trigPin = 14;                   //D5 is Trigger
const int echoPin = 12;                   //D6 is Echo
long duration;
long distance;
int trigState = LOW;                      //state of trigPin
bool alarmIsOn = false;

int hourAlarm = 18;
int minuteAlarm = 46;

void setup() 
{
  //Initializes serial communication, I2C, LCD, RTC, and Audio
  Serial.begin(115200);
  lcd.begin();
  lcd.backlight();  

  Blynk.begin(auth, ssid, pass);
 
  rtc.begin();
  rtc.adjust(DateTime(2020, 2, 1, 18, 45, 40)); //Set to this time and date for testing. If used properly, delete/comment this line

//  Catch the audio file, use I2S as output, and generate wav audio
  audioLogger = &Serial;
  file = new AudioFileSourcePROGMEM( viola, sizeof(viola) );
  out = new AudioOutputI2SNoDAC();
  wav = new AudioGeneratorWAV();
  

  //For Ultrasonic Sensor
  pinMode(trigPin, OUTPUT); // Sets the trigPin as an Output
  pinMode(echoPin, INPUT); // Sets the echoPin as an Input

  digitalWrite(trigPin, LOW);
}
 
void loop()
{
  unsigned long currentMillis = millis();       //Shows how many milliseconds has passed since the microcontroller got turned on

  //Displays the time on LCD
  if (currentMillis - previousMillis > interval )
  {
    DateTime now = rtc.now();                     //Shows the current time using the variable now
    previousMillis = currentMillis;  
    displayTime(now);
    if (now.hour() == hourAlarm && now.minute() == minuteAlarm && now.second() == 0)
    {
      //Plays the audio file in a loop
      if (!wav->isRunning()) wav->begin(file, out);
      alarmIsOn = true;
    } 
  }

  //Controls when the alarm goes off
  if (currentMillis - previousMillis2 > buttonInterval) 
  {
    alarmControl();
    previousMillis2 = currentMillis;
  }
    
  //Configures the ultrasonic sensor
  readDistance();  

  //Plays the alarm audio when it matches with the clock
  playAudioFile();
  
  //Stops the alarm
  if (distance != 0)
  {
    alarmStop();
  }    
    
  Blynk.run();
}

void displayTime(DateTime now)
{    
  lcd.setCursor(2,0);
  lcd.print("Jam: ");
  lcd.print(now.hour());       //Shows the hour
  lcd.print(":");
  if (now.minute() < 10) lcd.print(0);
  lcd.print(now.minute());     //Shows the minute
  lcd.print(":");
  if (now.second() < 10) lcd.print(0);
  lcd.print(now.second());     //Shows the second
  lcd.println();
  lcd.setCursor(2,1);
  lcd.print("Alarm: ");
  lcd.print(hourAlarm);
  lcd.print(":");
  if (minuteAlarm < 10) lcd.print(0);
  lcd.print(minuteAlarm);
  lcd.print(" ");
  
//  Serial.print(now.hour());
//  Serial.print(" ");
//  Serial.print(hourAlarm);
//  Serial.print(" ");
//  Serial.print(now.minute());
//  Serial.print(" ");
//  Serial.println(minuteAlarm);
}

void alarmControl()
{
  if (digitalRead(D3) == HIGH)
  {
    hourAlarm++;
    if (hourAlarm == 24)
      hourAlarm = 0;
  }
  if (digitalRead(D7) == HIGH)
  {
    hourAlarm--;
    if (hourAlarm == -1)
      hourAlarm = 23;
  }      
  if (digitalRead(D4) == HIGH)
  {
    minuteAlarm++;
    if (minuteAlarm == 60)
      minuteAlarm = 0;
  }      
  if (digitalRead(D8) == HIGH)
  {
    minuteAlarm--;
    if (minuteAlarm == -1)
      minuteAlarm = 59;
  }
}

void readDistance()
{
  unsigned long currentMicros = micros();       //Same function as millis() but in microseconds
  //Clears the trigPin for 2 microseconds, then sets it high
  if (currentMicros - previousMicros > triggerInterval1)
  {
    digitalWrite(trigPin, HIGH);    
  }
  
  // Sets the trigPin on HIGH state for 10 micro seconds
  // trigPin needs to be high at least 10 microseconds to send 8 pulses at 40 kHz, which will echo and be received back by the sensor
  if (currentMicros - previousMicros > triggerInterval2)
  {
    previousMicros = currentMicros;
    digitalWrite(trigPin, LOW);    
  }
  
  // Calculating the distance
  duration = read_pulse(echoPin);
  distance= duration*0.034/2; //in centimeter; 0.034 cm/us; divided by 2 because it bounces forward and backward
}

void alarmStop()
{  
  if (distance < 10 && distance != 0)
  {
     wav->stop();
     alarmIsOn = false;           
  }
}

void playAudioFile()
{
  if (wav->isRunning() && alarmIsOn == true) 
  {
    if (!wav->loop()) wav->stop();
//    Serial.println("L E M A O");
  } 
  else                                      //Deletes the audio file so that it can be looped
  {
    delete file; delete wav; 
    file = new AudioFileSourcePROGMEM( viola, sizeof(viola) );
    wav = new AudioGeneratorWAV();
    wav->begin(file, out);
  }
}

unsigned long read_pulse(int pin)
{
    static unsigned long rising_time;  // time of the rising edge
    static int last_state;             // previous pin state
    int state = digitalRead(pin);      // current pin state
    unsigned long pulse_length = 0;    // default return value

    // On rising edge: record current time.
    if (last_state == LOW && state == HIGH) {
        rising_time = micros();
    }

    // On falling edge: report pulse length.
    if (last_state == HIGH && state == LOW) {
        unsigned long falling_time = micros();
        pulse_length = falling_time - rising_time;
    }

    last_state = state;
    return pulse_length;
}
